#include "handling.hh"
#include "rom.hh"
#include "defs.hh"

unsigned BaseAddr      = 0;
unsigned BlockBaseAddr = 0;

class Blocks Blocks;

void Blocks::Load()
{
    Blocks& t = *this;
    resize(192);
    for(unsigned a=0; a<size(); ++a)
    {
        t[a].Load(BlockBaseAddr + Addr_Blocks + a*4,
                  BlockBaseAddr + Addr_BlockPals + a);
    }
}

unsigned Blocks::CreateOrFind(const Block& tmp)
{
    // Return the index of the block.
    // If already exists, return existing index.
    Blocks& t = *this;
    
    for(unsigned a=0; a<size(); ++a)
        if(t[a] == tmp)
            return a;
    
    if(size() >= 192)
    {
        fprintf(stderr, "Error: Too many blocks (%u)\n", size());
    }
    
    unsigned b_adr = BlockBaseAddr + Addr_Blocks    + size()*4;
    unsigned p_adr = BlockBaseAddr + Addr_BlockPals + size()*1;
    tmp.Write(b_adr, p_adr);
    
    ROM.DelFreeSpace(b_adr, 4);
    ROM.DelFreeSpace(p_adr, 1);
    
    push_back(tmp);
    return size()-1;
}

class Rooms Rooms;

void Rooms::Load()
{
    Rooms& t = *this;

    resize(32);
    for(unsigned a=0; a<size(); ++a)
        t[a].Load(BaseAddr + Addr_Pointers + a*2);
}

unsigned Rooms::CreateOrFind(const Room& tmp)
{
    // Return the index of the room.
    // If already exists, return existing index.
    Rooms& t = *this;
    for(unsigned a=0; a<size(); ++a)
        if(t[a] == tmp)
        {
            //printf("Repeat room %u\n", a);
            return a;
        }
    
    //printf("Writing room %u\n", size());
    
    if(Addr_Rooms + (size()+1) * 64 >= Addr_Screens)
    {
        fprintf(stderr, "Error: Too many rooms (%u)\n", size());
        return size();
    }
    
    tmp.Write(BaseAddr + Addr_Pointers + size()*2,
              BaseAddr + Addr_Rooms    + size()*64);
    
    push_back(tmp);
    return size()-1;
}
